
import unittest

from PyFoam.FoamInformation import oldTutorialStructure,foamTutorials,foamVersionNumber
from os import path,environ,system

from PyFoam.RunDictionary.ParsedParameterFile import FoamStringParser,ParsedParameterFile,ParsedBoundaryDict,DictProxy,TupleProxy,PyFoamParserError

from PyFoam.Basics.FoamFileGenerator import Vector,Dimension,Field,Tensor,SymmTensor

from PyFoam.FoamInformation import oldAppConvention as oldApp

from TimeDirectory import damBreakTutorial,gammaName

from PyFoam.Error import error

def simpleBikeTutorial():
    prefix=foamTutorials()
    if not oldTutorialStructure():
        prefix=path.join(prefix,"incompressible")
    else:
        error("The simpleFoam-motorBike-case does not exist before 1.6")
        
    return path.join(prefix,"simpleFoam","motorBike")

def simplePitzTutorial():
    prefix=foamTutorials()
    if not oldTutorialStructure():
        prefix=path.join(prefix,"incompressible")            
    return path.join(prefix,"simpleFoam","pitzDaily")

def buoyHotRoomTutorial():
    prefix=foamTutorials()
    if not oldTutorialStructure():
        prefix=path.join(prefix,"heatTransfer")            
    return path.join(prefix,"buoyantSimpleFoam","hotRoom")

def XoodlesPitzTutorial():
    prefix=foamTutorials()
    if oldTutorialStructure():
        prefix=path.join(prefix,"Xoodles")            
    else:
        prefix=path.join(prefix,"combustion","XiFoam","les")            
    return path.join(prefix,"pitzDaily3D")

def dieselAachenTutorial():
    prefix=foamTutorials()
    if not oldTutorialStructure():
        prefix=path.join(prefix,"combustion")            
    return path.join(prefix,"dieselFoam","aachenBomb")

def turbCavityTutorial():
    prefix=foamTutorials()
    if oldTutorialStructure():
        prefix=path.join(prefix,"turbFoam")            
    else:
        prefix=path.join(prefix,"incompressible","pisoFoam","ras")            
    return path.join(prefix,"cavity")
        
theSuite=unittest.TestSuite()
    
class FoamStringParserTest(unittest.TestCase):
    
    def testParseInteger(self):
        p1=FoamStringParser("test 1;")
        self.assertEqual(p1["test"],1)
        
    def testParseFloat(self):
        p1=FoamStringParser("test 1.23e-4;")
        self.assertAlmostEqual(p1["test"],1.234e-4,6)
        
    def testParseFloat2(self):
        p1=FoamStringParser("test 1.e-4;")
        self.assertEqual(type(p1["test"]),float)
        self.assertAlmostEqual(p1["test"],1.e-4,6)
        
    def testParseString(self):
        p1=FoamStringParser('test "der name";')
        self.assertEqual(p1["test"][1:-1],"der name")

    def testParseWord(self):
        p1=FoamStringParser('test  name;')
        self.assertEqual(p1["test"],"name")
    
    def testParseWordUniform(self):
        p1=FoamStringParser('test  uniform;')
        self.assertEqual(p1["test"],"uniform")
    
    def testParseWordMinus(self):
        p1=FoamStringParser('test  name-0;')
        self.assertEqual(p1["test"],"name-0")
    
    def testParseWordWithBrackets(self):
        p1=FoamStringParser('test  div((phi*nix),U);')
        self.assertEqual(p1["test"],"div((phi*nix),U)")
    
    def testParseWordWithBrackets2(self):
        p1=FoamStringParser('test  div((phi&nix),U);')
        self.assertEqual(p1["test"],"div((phi&nix),U)")
    
    def testParseStrangeNames(self):
        p1=FoamStringParser('test%1 1;')
        self.assertEqual(p1["test%1"],1)
    
    def testParseStrangeNames2(self):
        p1=FoamStringParser('test+1 1;')
        self.assertEqual(p1["test+1"],1)
    
    def testParseStrangeNames3(self):
        p1=FoamStringParser('test:1 1;')
        self.assertEqual(p1["test:1"],1)
    
    def testParseError(self):
        try:
            p1=FoamStringParser('test  name')
            self.fail()
        except PyFoamParserError:
            pass
        
    def testDataList(self):
        p1=FoamStringParser("test (2 3 4 5);")
        self.assertEqual(type(p1["test"]),list)
        self.assertEqual(len(p1["test"]),4)
        
    def testDataList2(self):
        p1=FoamStringParser("test (a b c d);")
        self.assertEqual(type(p1["test"]),list)
        self.assertEqual(len(p1["test"]),4)
        self.assertEqual(p1["test"][-1],"d")
        
    def testDataList3(self):
        p1=FoamStringParser("test (5 4 (2 3 4 5));")
        self.assertEqual(type(p1["test"]),list)
        self.assertEqual(len(p1["test"]),2)
        
    def testDataDictionary(self):
        p1=FoamStringParser("test { dings 2;}")
        self.assertEqual(type(p1["test"]),DictProxy)
        self.assertEqual(len(p1["test"]),1)

    def testEmptyDictionary(self):
        p1=FoamStringParser("test {}")
        self.assertEqual(len(p1["test"]),0)
        self.assertEqual(type(p1["test"]),DictProxy)

    def testDataTuple(self):
        p1=FoamStringParser("test dings 2;")
        self.assertEqual(type(p1["test"]),TupleProxy)
        self.assertEqual(len(p1["test"]),2)

    def testDataNone(self):
        p1=FoamStringParser("test;")
        self.assertEqual(p1["test"],None)

    def testDataVector(self):
        p1=FoamStringParser("test ( 1 3 4 );")
        self.assertEqual(p1["test"].__class__,Vector)
        
    def testDataTensor(self):
        p1=FoamStringParser("test ( 1 2 3 4 5 4 3 2 1 );")
        self.assertEqual(p1["test"].__class__,Tensor)
        
    def testDataSymmTensor(self):
        p1=FoamStringParser("test ( 1 2 3 4 5 4 );")
        self.assertEqual(p1["test"].__class__,SymmTensor)
        
    def testDataDimension(self):
        p1=FoamStringParser("test [1 2 -3 0 1.4 1 2];")
        self.assertEqual(p1["test"].__class__,Dimension)
        
        p1=FoamStringParser("test [1 2 -3 0 1.4];")
        self.assertEqual(p1["test"].__class__,Dimension)
        
    def testStringConversion(self):
        p1=FoamStringParser("test dings 2;")
        self.assertEqual(str(p1),"test dings     2 ;\n")

    def testOneLineComment(self):
        p1=FoamStringParser("// test dings 2;")
        self.assertEqual(str(p1),"")
        p1=FoamStringParser("""
        // test dings 2;
        test2 dings 3;""")
        self.assertEqual(str(p1),"test2 dings     3 ;\n")
        p1=FoamStringParser("test ding; // test dings 2;")
        self.assertEqual(p1["test"],"ding")
        
    def testMultiLineComment(self):
        p=FoamStringParser("""
        /* this should never be read
        This too */
        test dings 2;""")
        self.assertEqual(str(p),"test dings     2 ;\n")
        p=FoamStringParser("""
        /* this should never be read
        This too */
        test dings 2;
        /* Here goes the next comment */""")
        self.assertEqual(str(p),"test dings     2 ;\n")

    def testCommentOutElements(self):
        p1=FoamStringParser("""things (
// test dings 2;
nix // Hepp
);""")
        self.assertEqual(len(p1["things"]),1)
        self.assertEqual(p1["things"][0],"nix")

    def testListInList(self):
        p=FoamStringParser("test (2.3 (3.3 -3 3) 2);")
        self.assertEqual(len(p["test"]),3)
        self.assertEqual(type(p["test"][1]),Vector)
        self.assertEqual(p["test"][1][0],3.3)

    def testListAllPreList(self):
        p=FoamStringParser("test (3(1.1 -1 1) 3(2.2 -2 2) 3(3.3 -3 3));")
        self.assertEqual(len(p["test"]),3)
        self.assertEqual(type(p["test"][1]),Vector)
        self.assertEqual(p["test"][1][0],2.2)

    def testReactionList(self):
        p=FoamStringParser("""test (
someReaction
   0.5 H2 + O2 = H2O
   (1000 0 10)
otherReaction
   C + O2 = CO2
   ((1000 0 10) (3 4 5))
);""")
        self.assertEqual(type(p["test"][0]),str)
        self.assertEqual(type(p["test"][1]),str)
        self.assertEqual(type(p["test"][2]),Vector)
        self.assertEqual(type(p["test"][3]),str)
        self.assertEqual(type(p["test"][4]),str)
        self.assertEqual(type(p["test"][5]),list)

    def testFalseReactionBug(self):
        p1=FoamStringParser("""things (
test // test = dings 2;
nix
);""")
        self.assertEqual(len(p1["things"]),2)
        self.assertEqual(p1["things"][0],"test")

    def testStringWithAssignment(self):
        p1=FoamStringParser('''
nix "a=3+x;b=4;";
''')
        self.assertEqual(p1["nix"],'"a=3+x;b=4;"')
        
    def testDeletionOfEntry(self):
        p1=FoamStringParser("nix 2; test  3;")
        self.assertEqual(p1["nix"],2)
        del p1["nix"]
        self.assert_("nix" not in p1)

    def testSubstitute1(self):
        p1=FoamStringParser("nix $da;")
        self.assertEqual(p1["nix"],"$da")
        
    def testSubstitute2(self):
        p1=FoamStringParser("nix 2 $da;")
        self.assertEqual(p1["nix"],[2,"$da"])
        
    def testInclude(self):
        p1=FoamStringParser('#include "nixda"')
        
    def testRemove(self):
        p1=FoamStringParser('#remove da')
        
    def testRemoveList(self):
        p1=FoamStringParser('#remove (nix da)')
        
    def testInputMode(self):
        p1=FoamStringParser('#inputMode merge')
        
    def testInputMode2(self):
        p1=FoamStringParser('#inputMode overwrite')
        
    def testInputMode3(self):
        p1=FoamStringParser('#inputMode error')
        
    def testInputMode4(self):
        p1=FoamStringParser('#inputMode default')
        
theSuite.addTest(unittest.makeSuite(FoamStringParserTest,"test"))

class ParsedBoundaryDictTest(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.boundaryFile"
        system("cp "+path.join(simplePitzTutorial(),"constant","polyMesh","boundary")+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedBoundaryDict(self.theFile)
        self.assertEqual(len(test.content),5)
        self.assert_("inlet" in test)
        
theSuite.addTest(unittest.makeSuite(ParsedBoundaryDict,"test"))
                 
class ParsedParameterFileTest(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.turbulence"
        turb=path.join(simplePitzTutorial(),"constant")
        if oldApp():
            turb=path.join(turb,"turbulenceProperties")
        else:
            turb=path.join(turb,"RASProperties")

        system("cp "+turb+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedParameterFile(self.theFile)
        if foamVersionNumber()<(1,5):
            nrTurbModels=17
            model="turbulenceModel"
        else:
            if foamVersionNumber()<(1,6):            
                nrTurbModels=19
            else:
                nrTurbModels=3
                
            model="RASModel"
            
        self.assertEqual(len(test.content),nrTurbModels)
        self.assertEqual(test[model],"kEpsilon")
    
theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest,"test"))
                 
class ParsedParameterFileTest2(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test."+gammaName()
        system("cp "+path.join(damBreakTutorial(),"0",gammaName())+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedParameterFile(self.theFile)
        self.assertEqual(len(test["boundaryField"]),5)
    
theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest2,"test"))
                 
class ParsedParameterFileTest3(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.Thotroom"
        system("cp "+path.join(buoyHotRoomTutorial(),"0","T.org")+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedParameterFile(self.theFile)
        self.assertEqual(len(test["boundaryField"]),3)
        self.assertEqual(len(test["boundaryField"]["floor"]["value"].val),400)
    
theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest3,"test"))
                 
class ParsedParameterFileTest4(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.Xoodles.schems"
        system("cp "+path.join(XoodlesPitzTutorial(),"system","fvSchemes")+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedParameterFile(self.theFile)
        gradSchemes=1
        divSchemes=4
        if foamVersionNumber()<(1,6):
            gradSchemes=2
            divSchemes=5
        self.assertEqual(len(test["gradSchemes"]),gradSchemes)
        self.assertEqual(len(test["divSchemes"]["div(phi,ft_b_h_hu)"][2]),divSchemes)
    
theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest4,"test"))
                 
class ParsedParameterFileTest5(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.Rfile"
        system("cp "+path.join(turbCavityTutorial(),"0","R")+" "+self.theFile)

    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        test=ParsedParameterFile(self.theFile)
        self.assertEqual(test["internalField"].__class__,Field)
        self.assertEqual(test["internalField"].isUniform(),True)
        if foamVersionNumber()<(1,5):
            self.assertEqual(test["internalField"].value().__class__,Tensor)
        else:
            self.assertEqual(test["internalField"].value().__class__,SymmTensor)
            
theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest5,"test"))
                 
class ParsedParameterFileTest6(unittest.TestCase):
    def setUp(self):
        self.theFile="/tmp/test.diesleSpray"
        system("cp "+path.join(dieselAachenTutorial(),"0","spray")+" "+self.theFile)
        
    def tearDown(self):
        system("rm "+self.theFile)
        
    def testReadTutorial(self):
        try:
            test=ParsedParameterFile(self.theFile)
        except PyFoamParserError:
            pass
    
if foamVersionNumber()<(1,5):
    theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest6,"test"))
                 
class ParsedParameterFileIncludeTest(unittest.TestCase):
    def setUp(self):
        self.fileName=path.join(simpleBikeTutorial(),"0","U")

    def tearDown(self):
        pass

    def testBasicInclude(self):
        test=ParsedParameterFile(self.fileName,doMacroExpansion=True)
        self.assertEqual(test["pressure"],0)
        self.assertEqual("upperWall" in test["boundaryField"],True)
        self.assertEqual(test["boundaryField"]["upperWall"]["type"],"slip")
        
if foamVersionNumber()>=(1,6):
    theSuite.addTest(unittest.makeSuite(ParsedParameterFileIncludeTest,"test"))
    
